@@ -2,9 +2,7 @@
  *
  * rdf_internal.h - Redland RDF Application Framework internal API (header never shipped)
  *
- * $Id: rdf_internal.h 8534 2006-01-26 04:57:33Z cmdjb $
- *
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2000-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -59,7 +57,9 @@ void librdf_system_free(void *ptr);
 #define SYSTEM_FREE(ptr)   free(ptr)
 #endif
 
+#ifndef LIBRDF_ASSERT_DIE
 #define LIBRDF_ASSERT_DIE abort();
+#endif
 
 #else
 /* DEBUGGING TURNED OFF */
@@ -73,7 +73,9 @@ void librdf_system_free(void *ptr);
 #define SYSTEM_MALLOC(size)   malloc(size)
 #define SYSTEM_FREE(ptr)   free(ptr)
 
+#ifndef LIBRDF_ASSERT_DIE
 #define LIBRDF_ASSERT_DIE
+#endif
 
 #endif
 
@@ -91,7 +93,7 @@ void librdf_system_free(void *ptr);
   if(condition) { \
     LIBRDF_ASSERT_REPORT(msg) \
     LIBRDF_ASSERT_DIE \
-    return(ret); \
+    return ret; \
   } \
 } while(0)
 
@@ -107,7 +109,7 @@ void librdf_system_free(void *ptr);
   if(!pointer) { \
     LIBRDF_ASSERT_REPORT("object pointer of type " #type " is NULL.") \
     LIBRDF_ASSERT_DIE \
-    return(ret); \
+    return ret; \
   } \
 } while(0)
 
@@ -156,7 +158,6 @@ void librdf_sign_free(void *ptr);
 /* Fatal errors - always happen */
 #define LIBRDF_FATAL1(world, facility, message) librdf_fatal(world, facility, __FILE__, __LINE__ , __func__, message)
 
-#include <rdf_list_internal.h>
 #include <rdf_list.h>
 #include <rdf_files.h>
 #include <rdf_heuristics.h>
