@@ -2,9 +2,7 @@
  *
  * rdf_serializer_raptor.c - RDF Serializers via Raptor
  *
- * $Id: rdf_serializer_raptor.c 13149 2007-11-06 08:28:33Z laalto $
- *
- * Copyright (C) 2002-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2002-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -132,7 +130,7 @@ librdf_serializer_raptor_set_feature(void *context,
 {
   librdf_serializer_raptor_context* scontext=(librdf_serializer_raptor_context*)context;
   raptor_feature feature_i;
-  int value_i;
+  const unsigned char* value_s;
   
   if(!feature)
     return 1;
@@ -145,9 +143,10 @@ librdf_serializer_raptor_set_feature(void *context,
   if(!librdf_node_is_literal(value))
     return 1;
   
-  value_i=atoi((const char*)librdf_node_get_literal_value(value));
+  value_s=(const unsigned char*)librdf_node_get_literal_value(value);
 
-  return raptor_serializer_set_feature(scontext->rdf_serializer, feature_i, value_i);
+  return raptor_serializer_set_feature_string(scontext->rdf_serializer,
+                                              feature_i, value_s);
 }
 
 
@@ -335,7 +334,7 @@ librdf_serializer_raptor_serialize_stream_to_counted_string(void *context,
   iostr=raptor_new_iostream_to_string(&string, &string_length,
                                       malloc);
   if(!iostr) {
-    librdf_free_stream(stream);    
+    free(string);
     return NULL;
   }
     
@@ -344,6 +343,7 @@ librdf_serializer_raptor_serialize_stream_to_counted_string(void *context,
 
   if(rc) {
     raptor_free_iostream(iostr);
+    free(string);
     return NULL;
   }
     
