@@ -2,9 +2,7 @@
  *
  * rasqal_engine.c - Rasqal query engine
  *
- * $Id: rasqal_engine.c 13184 2007-11-13 07:42:53Z dajobe $
- *
- * Copyright (C) 2004-2007, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2004-2008, David Beckett http://www.dajobe.org/
  * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -43,6 +41,7 @@
 #include "rasqal_internal.h"
 
 
+
 /* local types */
 
 typedef enum {
@@ -72,7 +71,7 @@ static const char * rasqal_engine_step_names[STEP_LAST+1]={
 static void rasqal_free_engine_execution_data(rasqal_query* query, rasqal_query_results* query_results, void *data);
 static rasqal_engine_step rasqal_engine_check_constraint(rasqal_query *query, rasqal_graph_pattern *gp);
 static int rasqal_engine_graph_pattern_init(rasqal_query_results* query_results, rasqal_graph_pattern *gp);
-static int rasqal_engine_excute_next_lazy(rasqal_query_results *query_results);
+static int rasqal_engine_execute_next_lazy(rasqal_query_results *query_results);
 static int rasqal_engine_query_result_row_to_nodes(rasqal_query_results* query_results);
 
 
@@ -334,8 +333,11 @@ rasqal_engine_expand_wildcards(rasqal_query* rq)
     case RASQAL_QUERY_VERB_DELETE:
     case RASQAL_QUERY_VERB_INSERT:
     default:
-      rasqal_query_error(rq, "Cannot use wildcard * with query verb %s",
-                         rasqal_query_verb_as_string(rq->verb));
+      rq->failed=1;
+      rasqal_log_error_simple(rq->world, RAPTOR_LOG_LEVEL_ERROR,
+                              &rq->locator,
+                              "Cannot use wildcard * with query verb %s",
+                              rasqal_query_verb_as_string(rq->verb));
       rc=1;
       break;
   }
@@ -397,8 +399,10 @@ rasqal_engine_assign_variables(rasqal_query* rq)
         
         if(v == v2) {
           if(!warned) {
-            rasqal_query_warning(rq, "Variable %s duplicated in SELECT.", 
-                                 v->name);
+            rasqal_log_error_simple(rq->world, RAPTOR_LOG_LEVEL_WARNING,
+                                    &rq->locator,
+                                    "Variable %s duplicated in SELECT.", 
+                                    v->name);
             warned=1;
           }
           raptor_sequence_set_at(var_sequence, j, NULL);
@@ -458,12 +462,9 @@ rasqal_engine_assign_variables(rasqal_query* rq)
 }
   
 
-/* static */
-static rasqal_triples_source_factory Triples_Source_Factory;
-
-
 /**
  * rasqal_set_triples_source_factory:
+ * @world: rasqal_world object
  * @register_fn: registration function
  * @user_data: user data for registration
  *
@@ -474,13 +475,13 @@ static rasqal_triples_source_factory Triples_Source_Factory;
  *
  * The rasqal_triples_source_factory factory method new_triples_source is
  * called with the user data for some query and rasqal_triples_source.
- * 
+ *
  **/
 void
-rasqal_set_triples_source_factory(void (*register_fn)(rasqal_triples_source_factory *factory), void* user_data)
+rasqal_set_triples_source_factory(rasqal_world* world, void (*register_fn)(rasqal_triples_source_factory *factory), void* user_data)
 {
-  Triples_Source_Factory.user_data=user_data;
-  register_fn(&Triples_Source_Factory);
+  world->triples_source_factory.user_data=user_data;
+  register_fn(&world->triples_source_factory);
 }
 
 
@@ -488,6 +489,7 @@ rasqal_triples_source*
 rasqal_new_triples_source(rasqal_query_results* query_results)
 {
   rasqal_query* query=query_results->query;
+  rasqal_triples_source_factory* rtsf=&query->world->triples_source_factory;
   rasqal_triples_source* rts;
   int rc=0;
   
@@ -496,22 +498,27 @@ rasqal_new_triples_source(rasqal_query_results* query_results)
     return NULL;
 
   rts->user_data=RASQAL_CALLOC(user_data, 1,
-                               Triples_Source_Factory.user_data_size);
+                               rtsf->user_data_size);
   if(!rts->user_data) {
     RASQAL_FREE(rasqal_triples_source, rts);
     return NULL;
   }
   rts->query=query;
 
-  rc=Triples_Source_Factory.new_triples_source(query, 
-                                               Triples_Source_Factory.user_data,
-                                               rts->user_data, rts);
+  rc=rtsf->new_triples_source(query, 
+                              rtsf->user_data,
+                              rts->user_data, rts);
   if(rc) {
     query_results->failed=1;
+    query->failed=1;
     if(rc > 0) {
-      rasqal_query_error(query, "Failed to make triples source.");
+      rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
+                              &query->locator,
+                              "Failed to make triples source.");
     } else {
-      rasqal_query_error(query, "No data to query.");
+      rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
+                              &query->locator,
+                              "No data to query.");
     }
     RASQAL_FREE(user_data, rts->user_data);
     RASQAL_FREE(rasqal_triples_source, rts);
@@ -525,6 +532,8 @@ rasqal_new_triples_source(rasqal_query_results* query_results)
 void
 rasqal_free_triples_source(rasqal_triples_source *rts)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(rts, rasqal_triples_source);
+  
   if(rts->user_data) {
     rts->free_triples_source(rts->user_data);
     RASQAL_FREE(user_data, rts->user_data);
@@ -546,6 +555,8 @@ rasqal_triples_source_triple_present(rasqal_triples_source *rts,
 static void
 rasqal_free_triples_match(rasqal_triples_match* rtm)
 {
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(rtm, rasqal_triples_match);
+  
   rtm->finish(rtm, rtm->user_data);
   RASQAL_FREE(rasqal_triples_match, rtm);
 }
@@ -562,7 +573,8 @@ rasqal_new_triples_match(rasqal_query_results* query_results, void *user_data,
 
   rtm=(rasqal_triples_match *)RASQAL_CALLOC(rasqal_triples_match, 1,
                                             sizeof(rasqal_triples_match));
-  if(rtm)
+  if(rtm) {
+    rtm->world=query_results->query->world;
     if(query_results->triples_source->init_triples_match(rtm,
                                                  query_results->triples_source,
                                                  query_results->triples_source->user_data,
@@ -570,6 +582,7 @@ rasqal_new_triples_match(rasqal_query_results* query_results, void *user_data,
       rasqal_free_triples_match(rtm);
       rtm=NULL;
     }
+  }
 
   return rtm;
 }
@@ -692,8 +705,11 @@ rasqal_new_engine_gp_data(rasqal_graph_pattern* gp)
 static void
 rasqal_free_gp_data(rasqal_engine_gp_data* gp_data)
 {
-  rasqal_graph_pattern* gp=gp_data->gp;
+  rasqal_graph_pattern* gp;
 
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(gp_data, rasqal_engine_gp_data);
+  
+  gp=gp_data->gp;
   if(gp_data->triple_meta) {
     if(gp) {
       while(gp_data->column >= gp->start_column) {
@@ -802,9 +818,10 @@ rasqal_query_build_declared_in(rasqal_query* query)
                     v->name, i, column);
 #endif
     } else if(!v->expression)
-      rasqal_query_warning(query, 
-                           "Variable %s was selected but is unused in the query.", 
-                           v->name);
+      rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_WARNING,
+                              &query->locator,
+                              "Variable %s was selected but is unused in the query.", 
+                              v->name);
   }
 
 
@@ -835,9 +852,11 @@ rasqal_engine_group_graph_pattern_get_next_match(rasqal_query_results* query_res
 #endif
 
   /* FIXME - sequence of graph_patterns not implemented, finish */
-  rasqal_query_error(query,
-                     "Graph pattern %s operation is not implemented yet. Ending query execution.", 
-                     rasqal_graph_pattern_operator_as_string(gp->op));
+  query->failed=1;
+  rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
+                          &query->locator,
+                          "Graph pattern %s operation is not implemented yet. Ending query execution.", 
+                          rasqal_graph_pattern_operator_as_string(gp->op));
   
   RASQAL_DEBUG1("Failing query with sequence of graph_patterns\n");
   return 0;
@@ -911,9 +930,11 @@ rasqal_engine_triple_graph_pattern_get_next_match(rasqal_query_results* query_re
         /* Column has no triplesMatch so create a new query */
         m->triples_match=rasqal_new_triples_match(query_results, m, m, t);
         if(!m->triples_match) {
-          rasqal_query_error(query,
-                             "Failed to make a triple match for column%d",
-                             gp_data->column);
+          query->failed=1;
+          rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
+                                  &query->locator,
+                                  "Failed to make a triple match for column%d",
+                                  gp_data->column);
           /* failed to match */
           gp_data->column--;
           rc= -1;
@@ -1097,8 +1118,11 @@ rasqal_free_engine_execution_data(rasqal_query* query,
                                   rasqal_query_results* query_results,
                                   void *data)
 {
-  rasqal_engine_execution_data* execution_data=(rasqal_engine_execution_data*)data;
+  rasqal_engine_execution_data* execution_data;
 
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(data, rasqal_engine_execution_data);
+  
+  execution_data=(rasqal_engine_execution_data*)data;
   if(execution_data->seq)
     raptor_free_sequence(execution_data->seq);
   RASQAL_FREE(rasqal_engine_execution_data, execution_data);
@@ -1703,7 +1727,10 @@ rasqal_engine_get_next_result(rasqal_query_results *query_results)
   outergp=query->query_graph_pattern;
   if(!outergp || !outergp->graph_patterns) {
     /* FIXME - no graph patterns in query - end results */
-    rasqal_query_error(query, "No graph patterns in query. Ending query execution.");
+    query->failed=1;
+    rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
+                            &query->locator,
+                            "No graph patterns in query. Ending query execution.");
     query_results->finished=1;
     return 0;
   }
@@ -1711,7 +1738,10 @@ rasqal_engine_get_next_result(rasqal_query_results *query_results)
   graph_patterns_size=raptor_sequence_size(outergp->graph_patterns);
   if(!graph_patterns_size) {
     /* FIXME - no graph patterns in query - end results */
-    rasqal_query_error(query, "No graph patterns in query. Ending query execution.");
+    query->failed=1;
+    rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
+                            &query->locator,
+                            "No graph patterns in query. Ending query execution.");
     query_results->finished=1;
     return 0;
   }
@@ -1746,9 +1776,11 @@ rasqal_engine_get_next_result(rasqal_query_results *query_results)
 
     if(gp->graph_patterns) {
       /* FIXME - sequence of graph_patterns not implemented, finish */
-      rasqal_query_error(query,
-                         "Graph pattern %s operation is not implemented yet. Ending query execution.", 
-                         rasqal_graph_pattern_operator_as_string(gp->op));
+      query->failed=1;
+      rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
+                              &query->locator,
+                              "Graph pattern %s operation is not implemented yet. Ending query execution.", 
+                              rasqal_graph_pattern_operator_as_string(gp->op));
 
       RASQAL_DEBUG1("Failing query with sequence of graph_patterns\n");
       step=STEP_FINISHED;
@@ -2146,22 +2178,32 @@ int
 rasqal_engine_check_limit_offset(rasqal_query_results *query_results)
 {
   rasqal_query* query=query_results->query;
+  int limit;
+
+  if(!query)
+    return 0;
+
+  limit=query->limit;
+
+  /* Ensure ASK queries never do more than one result */
+  if(query->verb == RASQAL_QUERY_VERB_ASK)
+    limit=1;
 
   if(query->offset > 0) {
     /* offset */
     if(query_results->result_count <= query->offset)
       return -1;
     
-    if(query->limit >= 0) {
+    if(limit >= 0) {
       /* offset and limit */
-      if(query_results->result_count > (query->offset + query->limit)) {
+      if(query_results->result_count > (query->offset + limit)) {
         query_results->finished=1;
       }
     }
     
-  } else if(query->limit >= 0) {
+  } else if(limit >= 0) {
     /* limit */
-    if(query_results->result_count > query->limit) {
+    if(query_results->result_count > limit) {
       query_results->finished=1;
     }
   }
@@ -2586,9 +2628,9 @@ rasqal_engine_remove_empty_group_graph_patterns(rasqal_query* query,
  * Return value: non-0 on failure 
  */
 static int
-rasqal_engine_query_result_row_update(rasqal_query_result_row* row, int offset)
+rasqal_engine_query_result_row_update(rasqal_query_results *query_results,
+                                      rasqal_query_result_row* row, int offset)
 {
-  rasqal_query_results *query_results=row->results;
   rasqal_query* query;
   int i;
   int size;
@@ -2634,121 +2676,6 @@ rasqal_engine_query_result_row_update(rasqal_query_result_row* row, int offset)
 }
 
 
-/**
- * rasqal_engine_new_query_result_row:
- * @query_results: query results object
- * @offset: offset into sequence of results
- *
- * INTERNAL - Create a new query result row at an offset into the result sequence.
- *
- * Return value: a new query result row or NULL on failure
- */
-static rasqal_query_result_row*
-rasqal_engine_new_query_result_row(rasqal_query_results* query_results,
-                                   int offset)
-{
-  rasqal_query* query=query_results->query;
-  int size;
-  int order_size;
-  rasqal_query_result_row* row;
-  
-  if(!rasqal_query_results_is_bindings(query_results) &&
-     !rasqal_query_results_is_graph(query_results))
-    return NULL;
-
-  /* do not use rasqal_query_results_get_bindings_count() as it is 0
-   * for a graph result which is also executed handled here
-   */
-  if(query->constructs)
-    size=raptor_sequence_size(query->variables_sequence);
-  else
-    size=query->select_variables_count;
-
-  row=(rasqal_query_result_row*)RASQAL_CALLOC(rasqal_query_result_row, 1,
-                                              sizeof(rasqal_query_result_row));
-  if(!row)
-    return NULL;
-  row->usage=1;
-  row->results=query_results;
-
-  row->size=size;
-  row->values=(rasqal_literal**)RASQAL_CALLOC(array, size,
-					      sizeof(rasqal_literal*));
-  if(!row->values) {
-    rasqal_engine_free_query_result_row(row);
-    return NULL;
-  }
-
-  if(query->order_conditions_sequence)
-    order_size=raptor_sequence_size(query->order_conditions_sequence);
-  else
-    order_size=0;
-  
-  if(order_size) {
-    row->order_size=order_size;
-    row->order_values=(rasqal_literal**)RASQAL_CALLOC(array,  order_size,
-                                                      sizeof(rasqal_literal*));
-    if(!row->order_values) {
-      rasqal_engine_free_query_result_row(row);
-      return NULL;
-    }
-  }
-  
-  rasqal_engine_query_result_row_update(row, offset);
-  
-  return row;
-}
-
-
-/**
- * rasqal_engine_new_query_result_row_from_query_result_row:
- * @row: query result row
- * 
- * INTERNAL - Copy a query result row.
- *
- * Return value: a copy of the query result row or NULL
- */
-static rasqal_query_result_row*
-rasqal_engine_new_query_result_row_from_query_result_row(rasqal_query_result_row* row)
-{
-  row->usage++;
-  return row;
-}
-
-
-/**
- * rasqal_engine_free_query_result_row:
- * @row: query result row
- * 
- * INTERNAL - Free a query result row object.
- */
-void 
-rasqal_engine_free_query_result_row(rasqal_query_result_row* row)
-{
-  if(--row->usage)
-    return;
-  
-  if(row->values) {
-    int i; 
-    for(i=0; i < row->size; i++) {
-      if(row->values[i])
-        rasqal_free_literal(row->values[i]);
-    }
-    RASQAL_FREE(array, row->values);
-  }
-  if(row->order_values) {
-    int i; 
-    for(i=0; i < row->order_size; i++) {
-      if(row->order_values[i])
-        rasqal_free_literal(row->order_values[i]);
-    }
-    RASQAL_FREE(array, row->order_values);
-  }
-
-  RASQAL_FREE(rasqal_query_result_row, row);
-}
-
-
 /**
  * rasqal_engine_get_result_row:
  * @query_results: query results object
@@ -2820,60 +2747,6 @@ rasqal_engine_get_result_value(rasqal_query_results* query_results, int offset)
 }
 
 
-/**
- * rasqal_engine_query_result_row_print:
- * @row: query result row
- * @fp: FILE* handle
- *
- * INTERNAL - Print a query result row.
- */
-static void 
-rasqal_engine_query_result_row_print(rasqal_query_result_row* row, FILE* fh)
-{
-  rasqal_query* query=row->results->query;
-  int i;
-  
-  fputs("result[", fh);
-  for(i=0; i < row->size; i++) {
-    /* Do not use rasqal_query_results_get_binding_name(row->results, i); 
-     * as it does not work for a construct result
-     */
-    const unsigned char *name;
-    rasqal_literal *value;
-    
-    name=(query->variables[i] ? query->variables[i]->name : NULL);
-    value=row->values[i];
-    if(i > 0)
-      fputs(", ", fh);
-    fprintf(fh, "%s=", name);
-
-    if(value)
-      rasqal_literal_print(value, fh);
-    else
-      fputs("NULL", fh);
-  }
-
-  fputs(" with ordering values [", fh);
-
-  if(row->order_size) {
-
-    for(i=0; i < row->order_size; i++) {
-      rasqal_literal *value=row->order_values[i];
-      
-      if(i > 0)
-        fputs(", ", fh);
-      if(value)
-        rasqal_literal_print(value, fh);
-      else
-        fputs("NULL", fh);
-    }
-    fputs("]", fh);
-  }
-
-  fprintf(fh, " offset %d]", row->offset);
-}
-
-
 /**
  * rasqal_query_result_literal_sequence_compare:
  * @query: the #rasqal_query to use to comparekfind the variables in
@@ -3018,14 +2891,13 @@ rasqal_engine_query_result_row_compare(const void *a, const void *b)
 {
   rasqal_query_result_row* row_a;
   rasqal_query_result_row* row_b;
-  rasqal_query_results* results;
   rasqal_query* query;
   int result=0;
 
   row_a=*(rasqal_query_result_row**)a;
   row_b=*(rasqal_query_result_row**)b;
-  results=row_a->results;
-  query=results->query;
+
+  query=rasqal_rowsource_get_query(row_a->rowsource);
   
   if(query->distinct) {
     if(query->distinct == 1)
@@ -3062,9 +2934,9 @@ static void
 rasqal_engine_map_free_query_result_row(const void *key, const void *value)
 {
   if(key)
-    rasqal_engine_free_query_result_row((rasqal_query_result_row*)key);
+    rasqal_free_query_result_row((rasqal_query_result_row*)key);
   if(value)
-    rasqal_engine_free_query_result_row((rasqal_query_result_row*)value);
+    rasqal_free_query_result_row((rasqal_query_result_row*)value);
 }
 
 
@@ -3072,7 +2944,7 @@ static void
 rasqal_engine_map_print_query_result_row(void *object, FILE *fh)
 {
   if(object)
-    rasqal_engine_query_result_row_print((rasqal_query_result_row*)object, fh);
+    rasqal_query_result_row_print((rasqal_query_result_row*)object, fh);
   else
     fputs("NULL", fh);
 }
@@ -3082,7 +2954,7 @@ static void
 rasqal_engine_map_add_to_sequence(void *key, void *value, void *user_data)
 {
   rasqal_query_result_row* row;
-  row=rasqal_engine_new_query_result_row_from_query_result_row((rasqal_query_result_row*)key);
+  row=rasqal_new_query_result_row_from_query_result_row((rasqal_query_result_row*)key);
   raptor_sequence_push((raptor_sequence*)user_data, row);
 }
 
@@ -3093,6 +2965,9 @@ rasqal_engine_bind_construct_variables(rasqal_query_results* query_results)
   rasqal_query *query=query_results->query;
   int i;
 
+  if(!query)
+    return;
+  
   if(!query->constructs)
     return;
   
@@ -3102,11 +2977,242 @@ rasqal_engine_bind_construct_variables(rasqal_query_results* query_results)
   /* bind the construct variables again if running through a sequence */
   for(i=0; i< query->variables_count; i++) {
     rasqal_literal* value=rasqal_engine_get_result_value(query_results, i);
-    rasqal_variable_set_value(query->variables[i],  value);
+    rasqal_variable_set_value(query->variables[i],  rasqal_new_literal_from_literal(value));
   }
 }
 
 
+typedef struct 
+{
+  rasqal_query_results* results;
+  rasqal_map* map;
+  raptor_sequence* seq;
+  int need_store_results;
+  int finished;
+  int failed;
+  int offset;
+} rasqal_rowsource_engine_context;
+
+/* Local handlers for getting rows from a query execution */
+
+static int
+rasqal_rowsource_engine_init(rasqal_rowsource* rowsource, void *user_data) 
+{
+  rasqal_rowsource_engine_context* con=(rasqal_rowsource_engine_context*)user_data;
+  con->offset=0;
+  return 0;
+}
+
+
+static int
+rasqal_rowsource_engine_finish(rasqal_rowsource* rowsource, void *user_data)
+{
+  rasqal_rowsource_engine_context* con=(rasqal_rowsource_engine_context*)user_data;
+  if(con->map)
+    rasqal_free_map(con->map);
+  if(con->seq)
+    raptor_free_sequence(con->seq);
+  RASQAL_FREE(rasqal_rowsource_engine_context, con);
+
+  return 0;
+}
+
+
+static void
+rasqal_rowsource_engine_process(rasqal_rowsource* rowsource,
+                                rasqal_rowsource_engine_context* con,
+                                int read_all)
+{
+  if(con->finished || con->failed)
+    return;
+  
+  while(1) {
+    rasqal_query_result_row* row;
+    int rc;
+
+    /* query_results->results_sequence is NOT assigned before here 
+     * so that this function does the regular query results next
+     * operation.
+     */
+    rc=rasqal_engine_get_next_result(con->results);
+    if(rc == 0) {
+      /* =0 end of results */
+      con->finished=1;
+      break;
+    }
+    
+    if(rc < 0) {
+      /* <0 failure */
+      con->results->finished=1;
+      con->results->failed=1;
+      
+      if(con->map) {
+        rasqal_free_map(con->map);
+        con->map=NULL;
+      }
+      raptor_free_sequence(con->seq);
+      con->seq=NULL;
+      break;
+    }
+    
+    /* otherwise is >0 match */
+    row=rasqal_new_query_result_row(rowsource);
+    if(!row) {
+      raptor_free_sequence(con->seq); con->seq=NULL;
+      if(con->map) {
+        rasqal_free_map(con->map); con->map=NULL;
+      }
+      con->failed=1;
+      return;
+    }
+    
+    rasqal_engine_query_result_row_update(con->results, row, con->offset);
+
+    if(!con->map) {
+      /* no map. after this, row is owned by sequence */
+      raptor_sequence_push(con->seq, row);
+      con->offset++;
+    } else {
+      /* map. after this, row is owned by map */
+      if(!rasqal_map_add_kv(con->map, row, NULL)) {
+        con->offset++;
+      } else {
+        /* duplicate, and not added so delete it */
+#ifdef RASQAL_DEBUG
+        RASQAL_DEBUG1("Got duplicate row ");
+        rasqal_query_result_row_print(row, stderr);
+        fputc('\n', stderr);
+#endif
+        rasqal_free_query_result_row(row);
+        row=NULL;
+      }
+    }
+
+    /* if a row was returned and not storing result, end loop */
+    if(!read_all && !con->need_store_results)
+      return;
+  }
+  
+#ifdef RASQAL_DEBUG
+  if(con->map) {
+    fputs("resulting ", stderr);
+    rasqal_map_print(con->map, stderr);
+    fputs("\n", stderr);
+  }
+#endif
+  
+  if(con->map) {
+    /* do sort/distinct: walk map in order, adding rows to sequence */
+    rasqal_map_visit(con->map, rasqal_engine_map_add_to_sequence,
+                     (void*)con->seq);
+    rasqal_free_map(con->map); con->map=NULL;
+  }
+}
+
+
+static int
+rasqal_rowsource_engine_ensure_variables(rasqal_rowsource* rowsource,
+                                         void *user_data)
+{
+  rasqal_rowsource_engine_context* con=(rasqal_rowsource_engine_context*)user_data;
+  rasqal_query* query=con->results->query;
+
+  if(query->constructs)
+    rowsource->size=raptor_sequence_size(query->variables_sequence);
+  else
+    rowsource->size=query->select_variables_count;  
+  if(query->order_conditions_sequence)
+    rowsource->order_size=raptor_sequence_size(query->order_conditions_sequence);
+
+  return 0;
+}
+
+static rasqal_query_result_row*
+rasqal_rowsource_engine_read_row(rasqal_rowsource* rowsource, void *user_data)
+{
+  rasqal_rowsource_engine_context* con=(rasqal_rowsource_engine_context*)user_data;
+  if(!con->finished)
+    rasqal_rowsource_engine_process(rowsource, con, 0);
+
+  return (rasqal_query_result_row*)raptor_sequence_unshift(con->seq);
+}
+
+
+static raptor_sequence*
+rasqal_rowsource_engine_read_all_rows(rasqal_rowsource* rowsource,
+                                      void *user_data)
+{
+  rasqal_rowsource_engine_context* con=(rasqal_rowsource_engine_context*)user_data;
+  raptor_sequence* seq;
+  
+  if(!con->finished)
+    rasqal_rowsource_engine_process(rowsource, con, 1);
+  
+  seq=con->seq;
+  con->seq=NULL;
+  
+  return seq;
+}
+
+
+static rasqal_query*
+rasqal_rowsource_engine_get_query(rasqal_rowsource* rowsource, void *user_data)
+{
+  rasqal_rowsource_engine_context* con=(rasqal_rowsource_engine_context*)user_data;
+  return con->results->query;
+}
+
+
+static const rasqal_rowsource_handler rasqal_rowsource_engine_handler={
+  /* .version = */ 1,
+  /* .init = */ rasqal_rowsource_engine_init,
+  /* .finish = */ rasqal_rowsource_engine_finish,
+  /* .ensure_variables = */ rasqal_rowsource_engine_ensure_variables,
+  /* .read_row = */ rasqal_rowsource_engine_read_row,
+  /* .read_all_rows = */ rasqal_rowsource_engine_read_all_rows,
+  /* .get_query = */ rasqal_rowsource_engine_get_query
+};
+
+static rasqal_rowsource*
+rasqal_engine_make_rowsource(rasqal_query* query, rasqal_query_results* results) 
+{
+  rasqal_rowsource_engine_context* con;
+  int flags=0;
+  
+  con=(rasqal_rowsource_engine_context*)RASQAL_CALLOC(rasqal_rowsource_engine_context, 1, sizeof(rasqal_rowsource_engine_context));
+  if(!con)
+    return NULL;
+
+  con->results=results;
+
+  con->need_store_results=(query->order_conditions_sequence || query->distinct);
+
+  if(con->need_store_results) {
+    /* make a row:NULL map in order to sort or do distinct */
+    con->map=rasqal_new_map(rasqal_engine_query_result_row_compare,
+                            rasqal_engine_map_free_query_result_row, 
+                            rasqal_engine_map_print_query_result_row,
+                            NULL,
+                            0);
+    if(!con->map) {
+      rasqal_rowsource_engine_finish(NULL, con);
+      return NULL;
+    }
+  }
+  
+  con->seq=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_query_result_row, (raptor_sequence_print_handler*)rasqal_query_result_row_print);
+  if(!con->seq) {
+    rasqal_rowsource_engine_finish(NULL, con);
+    return NULL;
+  }
+  
+  flags=con->need_store_results ? RASQAL_ROWSOURCE_FLAGS_ORDERING : 0;
+
+  return rasqal_new_rowsource_from_handler(con,
+                                           &rasqal_rowsource_engine_handler,
+                                           flags);
+}
+
 /**
  * rasqal_engine_execute_and_save:
  * @query_results: Query results to execute
@@ -3120,87 +3226,18 @@ rasqal_engine_execute_and_save(rasqal_query_results *query_results)
 {
   rasqal_query *query=query_results->query;
   int rc=0;
-  rasqal_map* map=NULL;
-  raptor_sequence* seq;
-  int offset=0;
+
+  if(query_results->results_sequence)
+     raptor_free_sequence(query_results->results_sequence);
+
+  query_results->results_sequence=rasqal_rowsource_read_all_rows(query_results->rowsource);
+
+  rasqal_free_rowsource(query_results->rowsource);
+  query_results->rowsource=NULL;
   
-  /* make a row:NULL map */
-  map=rasqal_new_map(rasqal_engine_query_result_row_compare,
-                     rasqal_engine_map_free_query_result_row, 
-                     rasqal_engine_map_print_query_result_row,
-                     NULL,
-                     0);
-  if(!map)
-    return -1;
-  
-  /* get all query results and order them */
-  seq=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_engine_free_query_result_row, (raptor_sequence_print_handler*)rasqal_engine_query_result_row_print);
-  if(!seq) {
-    rasqal_free_map(map);
-    return -1;
-  }
-  while(1) {
-    rasqal_query_result_row* row;
-    
-    /* query_results->results_sequence is NOT assigned before here 
-     * so that this function does the regular query results next
-     * operation.
-     */
-    rc=rasqal_engine_get_next_result(query_results);
-    if(rc == 0)
-      /* <0 failure OR =0 end of results */
-      break;
-    
-    if(rc < 0) {
-      /* <0 failure */
-      query_results->finished=1;
-      query_results->failed=1;
-      
-      if(map)
-        rasqal_free_map(map);
-      raptor_free_sequence(seq);
-      seq=NULL;
-      break;
-    }
-    
-    /* otherwise is >0 match */
-    row=rasqal_engine_new_query_result_row(query_results, offset);
-    if(!row) {
-      raptor_free_sequence(seq);
-      rasqal_free_map(map);
-      return -1;
-    }
-    
-    /* after this, row is owned by map */
-    if(!rasqal_map_add_kv(map, row, NULL)) {
-      offset++;
-    } else {
-      /* duplicate, and not added so delete it */
-#ifdef RASQAL_DEBUG
-      RASQAL_DEBUG1("Got duplicate row ");
-      rasqal_engine_query_result_row_print(row, stderr);
-      fputc('\n', stderr);
-#endif
-      rasqal_engine_free_query_result_row(row);
-      row=NULL;
-    }
-  }
-  
-#ifdef RASQAL_DEBUG
-  if(map) {
-    fputs("resulting ", stderr);
-    rasqal_map_print(map, stderr);
-    fputs("\n", stderr);
-  }
-#endif
-  
-  if(map) {
-    rasqal_map_visit(map, rasqal_engine_map_add_to_sequence, (void*)seq);
-    rasqal_free_map(map);
-  }
-  query_results->results_sequence=seq;
-  
-  if(query_results->results_sequence) {
+  if(!query_results->results_sequence) {
+    query_results->finished=1;
+  } else {
     query_results->finished= (raptor_sequence_size(query_results->results_sequence) == 0);
     
     if(!query->limit)
@@ -3250,7 +3287,7 @@ rasqal_engine_execute_and_save(rasqal_query_results *query_results)
  *
  * When results are not stored, query->results_sequence is NULL and
  * only the first result is calculated using
- * rasqal_engine_excute_next_lazy()
+ * rasqal_engine_execute_next_lazy()
  *
  * Return value: <0 if failed, 0 if result, >0 if finished
  */
@@ -3259,11 +3296,21 @@ rasqal_engine_execute_run(rasqal_query_results* query_results)
 {
   rasqal_query *query=query_results->query;
   int rc=0;
+
+  if(query_results->rowsource)
+    rasqal_free_rowsource(query_results->rowsource);
   
-  if(query->order_conditions_sequence || query->distinct)
+  query_results->rowsource=rasqal_engine_make_rowsource(query, query_results);
+  if(!query_results->rowsource) {
+    query_results->finished=1;
+    return 1;
+  }
+  
+  if(query->store_results || 
+     query->order_conditions_sequence || query->distinct)
     rc=rasqal_engine_execute_and_save(query_results);
   else
-    rc=rasqal_engine_excute_next_lazy(query_results);
+    rc=rasqal_engine_execute_next_lazy(query_results);
 
   if(rc >= 0)
     rc=rasqal_engine_query_result_row_to_nodes(query_results);
@@ -3349,7 +3396,7 @@ rasqal_engine_execute_next_from_saved(rasqal_query_results* query_results)
 
 
 /**
- * rasqal_engine_excute_next_lazy:
+ * rasqal_engine_execute_next_lazy:
  * @query_results: Query results to execute
  *
  * INTERNAL - Execute a query to get one result, finished or failure.
@@ -3357,7 +3404,7 @@ rasqal_engine_execute_next_from_saved(rasqal_query_results* query_results)
  * Return value: <0 if failed, 0 if result, >0 if finished
  */
 static int
-rasqal_engine_excute_next_lazy(rasqal_query_results *query_results)
+rasqal_engine_execute_next_lazy(rasqal_query_results *query_results)
 {
   if(!query_results)
     return -1;
@@ -3406,10 +3453,10 @@ rasqal_engine_excute_next_lazy(rasqal_query_results *query_results)
 
   if(!query_results->finished) {
     if(!query_results->row)
-      query_results->row=rasqal_engine_new_query_result_row(query_results, 
-                                                            query_results->result_count);
-    else
-      rasqal_engine_query_result_row_update(query_results->row, 
+      query_results->row=rasqal_new_query_result_row(query_results->rowsource);
+
+    if(query_results->row)
+      rasqal_engine_query_result_row_update(query_results, query_results->row, 
                                             query_results->result_count);
   }
   
@@ -3430,7 +3477,7 @@ rasqal_engine_excute_next_lazy(rasqal_query_results *query_results)
  * When a results sequence exists, the next result is pulled
  * from the sequence, checking any limit and offset here.
  *
- * When evaluating lazily, rasqal_engine_excute_next_lazy()
+ * When evaluating lazily, rasqal_engine_execute_next_lazy()
  * Is called to initialise the state for the next result.
  */
 int
@@ -3439,7 +3486,7 @@ rasqal_engine_execute_next(rasqal_query_results* query_results)
   if(query_results->results_sequence)
     rasqal_engine_execute_next_from_saved(query_results);
   else
-    rasqal_engine_excute_next_lazy(query_results);
+    rasqal_engine_execute_next_lazy(query_results);
 
   rasqal_engine_query_result_row_to_nodes(query_results);
   
